import path from "path";
import vue from 'rollup-plugin-vue';
import alias from "@rollup/plugin-alias";
import commonjs from "@rollup/plugin-commonjs";
import typescript from 'rollup-plugin-typescript2';
import sass from 'rollup-plugin-scss';
import copy from 'rollup-plugin-copy';
import PostCSS from "rollup-plugin-postcss";
import simplevars from "postcss-simple-vars";
import postcssImport from "postcss-import";
import postcssUrl from "postcss-url";
import url from "@rollup/plugin-url";
import nested from "postcss-nested";
import autoprefixer from "autoprefixer";
import css from 'rollup-plugin-css-only';
import resolve from "@rollup/plugin-node-resolve";
import terser from '@rollup/plugin-terser';

import pkg from "./package.json" assert {type: "json"}

const postcssConfigList = [
  postcssImport({
    resolve(id, basedir) {
      // resolve alias @css, @import '@css/style.css'
      // because @css/ has 5 chars
      if (id.startsWith("@css")) {
        return path.resolve("./src/assets/styles/css", id.slice(5));
      }

      // resolve node_modules, @import '~normalize.css/normalize.css'
      // similar to how css-loader's handling of node_modules
      if (id.startsWith("~")) {
        return path.resolve("./node_modules", id.slice(1));
      }

      // resolve relative path, @import './components/style.css'
      return path.resolve(basedir, id);
    }
  }),
  simplevars,
  nested,
  postcssUrl({ url: "inline" }),
  autoprefixer({
    overrideBrowserslist: "> 1%, IE 6, Explorer >= 10, Safari >= 7"
  })
];

const projectRoot = path.resolve( ".");

let postVueConfig = [
  // Process only `<style module>` blocks.
  PostCSS({
    modules: {
      generateScopedName: '[local]___[hash:base64:5]',
    },
    include: /&module=.*\.css$/,
  }),
  // Process all `<style>` blocks except `<style module>`.
  PostCSS({ include: /(?<!&module=.*)\.css$/,
    plugins:[
      ...postcssConfigList
    ]
   }),
  url({
      include: [
        '**/*.svg',
        '**/*.png',
        '**/*.gif',
        '**/*.jpg',
        '**/*.jpeg'
      ]
    }),
]

if(process.env.SEP_CSS){
  postVueConfig = [css({ output: './dist/bundle.css' }), ...postVueConfig]
}

const baseConfig = {
  plugins: {
    preVue: [
      alias({
        entries: [
          {
            find: "@",
            replacement: `${path.resolve(projectRoot, "src")}`
          }
        ],
        customResolver: resolve({
          extensions: [".js", ".jsx", ".vue"]
        })
      })
    ],
    replace: {
      "process.env.NODE_ENV": JSON.stringify("production"),
      __VUE_OPTIONS_API__: JSON.stringify(true),
      __VUE_PROD_DEVTOOLS__: JSON.stringify(false)
    },
    vue: {
      target: "browser",
      preprocessStyles: process.env.SEP_CSS ? false : true,
      postcssPlugins: [...postcssConfigList],
      css: true
    },
    postVue: [
      ...postVueConfig
    ]
  }
};

export default {
  input: 'src/index.ts',
  output: [
    {
      format: "cjs",
      file: `dist/${pkg.name}-min.cjs`,
      sourcemap: true
    },
    {
      format: "esm",
      file: `dist/${pkg.name}-min.mjs`,
      sourcemap: true
    },
  ],
  plugins: [
    typescript(),
    ...baseConfig.plugins.preVue,
    vue({
      ...baseConfig.plugins.vue,
      template: {
        ...baseConfig.plugins.vue.template,
      }
    }),
    ...baseConfig.plugins.postVue,
    process.env.NODE_ENV === 'production' ? terser() : undefined,
    commonjs(),
    sass(),
    copy({
      targets: [
        { src: './CHANGELOG.md', dest: './dist/' },
      ],
    }),
  ],
};

